@@ -201,15 +201,20 @@ myreturn (struct command *cmd)	/* return from gosub of function call */
 void
 create_subr_link (char *label)	/* create link to subroutine */
 {
-    char global[200];
+    char global[NAMEBUFFLEN];
     char *dot;
     struct command *cmd;
 
     if (!inlib) {
-	error(sDEBUG, "not in library, will not create link to subroutine");
+	if (severity_threshold <= sDEBUG)
+	    error (sDEBUG, "not in library, will not create link to subroutine");
         return;
     }
     dot = strchr (label, '.');
+    if (strlen (library_stack[include_depth-1]->short_name) + strlen (dot) >= NAMEBUFFLEN) {
+	error (sERROR, "Name of function too long");
+	return;
+    }
     strcpy (global, library_stack[include_depth-1]->short_name);
     strcat (global, dot);
 
@@ -249,8 +254,8 @@ function_or_array (struct command *cmd)	/* decide whether to perform function or
         cmd->pointer = cmd->symname;
         cmd->symname = NULL;
 	if (severity_threshold <= sDEBUG) {
-	    sprintf(errorstring, "converting '%s' to '%s'",explanation[cFUNCTION_OR_ARRAY],explanation[cFUNCTION]);
-	    error(sDEBUG, errorstring);
+	    sprintf(estring, "converting '%s' to '%s'",cexplanation[cFUNCTION_OR_ARRAY],cexplanation[cFUNCTION]);
+	    error(sDEBUG, estring);
 	}
     } else {
         if (cmd->type == cFUNCTION_OR_ARRAY) {
@@ -261,8 +266,8 @@ function_or_array (struct command *cmd)	/* decide whether to perform function or
         cmd->type = cDOARRAY;
         cmd->args = -1;
 	if ( severity_threshold <= sDEBUG) {
-	    sprintf(errorstring, "converting '%s' to '%s'",explanation[cFUNCTION_OR_ARRAY],explanation[cDOARRAY]);
-	    error(sDEBUG, errorstring);
+	    sprintf(estring, "converting '%s' to '%s'",cexplanation[cFUNCTION_OR_ARRAY],cexplanation[cDOARRAY]);
+	    error(sDEBUG, estring);
 	}
     }
 }
@@ -698,7 +703,7 @@ pop_multi (struct command *cmd) /* pop and discard multiple values from stack */
 void load_pop_multi (struct command *cmd, int to_pop) /* put correct value into preceding pop_multi-statement */
 {
     if (cmd->prev->type != cPOP_MULTI) {
-	sprintf(string, "while trying to load pop_multi; preceding command is rather '%s'", explanation[cmd->prev->type]);
+	sprintf(string, "while trying to load pop_multi; preceding command is rather '%s'", cexplanation[cmd->prev->type]);
 	error(sFATAL, string);
     }
     cmd->prev->tag = to_pop;
@@ -758,14 +763,14 @@ mybreak (struct command *cmd)	/* find break_here statement */
 	curr = curr->next;
 	if (!curr) {
 	    curr = cmd;
-	    sprintf(errorstring,"break has left program (loop_nesting=%d, switch_nesting=%d)",loop_nesting,switch_nesting);
-	    error (sERROR, errorstring);
+	    sprintf(estring,"break has left program (loop_nesting=%d, switch_nesting=%d)",loop_nesting,switch_nesting);
+	    error (sERROR, estring);
 	}
     }
     cmd->type = cQGOTO;
     if (severity_threshold <= sDEBUG) {
-	sprintf(errorstring, "converting '%s' to '%s'",explanation[cBREAK_MULTI],explanation[cQGOTO]);
-        error (sDEBUG, errorstring);
+	sprintf(estring, "converting '%s' to '%s'",cexplanation[cBREAK_MULTI],cexplanation[cQGOTO]);
+        error (sDEBUG, estring);
     }
     load_pop_multi(cmd, to_pop);
     cmd->jump = current = curr;
@@ -803,8 +808,8 @@ mycontinue (struct command *cmd)	/* find continue_here statement */
     }
     cmd->type = cQGOTO;
     if (severity_threshold <= sDEBUG) {
-	sprintf( errorstring, "converting '%s' to '%s'",explanation[cCONTINUE],explanation[cQGOTO]);
-	error (sDEBUG, errorstring);
+	sprintf( estring, "converting '%s' to '%s'",cexplanation[cCONTINUE],cexplanation[cQGOTO]);
+	error (sDEBUG, estring);
     }
     load_pop_multi(cmd, to_pop);
     cmd->jump = current = curr;
@@ -835,14 +840,14 @@ next_case (struct command *cmd)	/* find next_case_here statement */
         curr = curr->next;
         if (!curr) {
 	    curr = cmd;
-	    sprintf(errorstring,"search for next case has left program (loop_nesting=%d, switch_nesting=%d)",loop_nesting,switch_nesting);
-	    error (sERROR, errorstring);
+	    sprintf(estring,"search for next case has left program (loop_nesting=%d, switch_nesting=%d)",loop_nesting,switch_nesting);
+	    error (sERROR, estring);
         }
     }
     cmd->type = cQGOTO;
     if (severity_threshold <= sDEBUG) {
-	sprintf(errorstring,"converting '%s' to '%s'",explanation[cNEXT_CASE],explanation[cQGOTO]);
-        error (sDEBUG, errorstring);
+	sprintf(estring,"converting '%s' to '%s'",cexplanation[cNEXT_CASE],cexplanation[cQGOTO]);
+        error (sDEBUG, estring);
     }
     cmd->jump = current = curr;
 }
