@@ -375,6 +375,10 @@ create_function (int type)	/* create command 'function' */
     struct command *cmd;
 
     cmd = add_command (cFUNCTION, FALSE, NULL);
+    if (severity_threshold <= sDEBUG) {
+	sprintf(estring, "function '%s'",fexplanation[type]);
+	error (sDEBUG, estring);
+    }
     cmd->args = type;
 }
 
@@ -404,6 +408,11 @@ function (struct command *current)	/* performs a function */
         a1 = pop (stSTRING_OR_NUMBER);
     }
 
+    if (severity_threshold <= sDEBUG) {
+	sprintf(estring, "function '%s'",fexplanation[type]);
+	error (sDEBUG, estring);
+    }
+
     switch (type) {
     case fSIN:
         value = sin (a1->value);
@@ -775,7 +784,7 @@ function (struct command *current)	/* performs a function */
         time (&datetime);
         strftime (pointer, 100, "%H-%M-%S", localtime (&datetime));
         sprintf (pointer + strlen (pointer), "-%d",
-                 (int) (time (NULL) - compilation_start));
+                 (int) ((current_millis () - compilation_start)/1000));
         result = stSTRING;
         break;
     case fSYSTEM:
@@ -788,6 +797,43 @@ function (struct command *current)	/* performs a function */
         pointer = do_system2 (str);
         result = stSTRING;
         break;
+    case fFRNFN_CALL:
+	frnfn_call (type, &value, &pointer);
+        result = stNUMBER;
+        break;
+    case fFRNFN_CALL2:
+	frnfn_call (type, &value, &pointer);
+        result = stSTRING;
+        break;
+    case fFRNBF_ALLOC:
+	pointer = frnbf_alloc ();
+        result = stSTRING;
+        break;
+    case fFRNBF_SIZE:
+	value = frnbf_size ();
+	result = stNUMBER;
+        break;
+    case fFRNBF_DUMP:
+    case fFRNBF_DUMP2:
+	pointer = frnbf_dump (type);
+        result = stSTRING;
+        break;
+    case fFRNFN_SIZE:
+	value = frnfn_size ();
+        result = stNUMBER;
+        break;
+    case fFRNBF_GET_NUMBER:
+	value = frnbf_get ();
+        result = stNUMBER;
+        break;
+    case fFRNBF_GET_STRING:
+	pointer = frnbf_get2 ();
+        result = stSTRING;
+        break;
+    case fFRNBF_GET_BUFFER:
+	pointer = frnbf_get_buffer ();
+        result = stSTRING;
+        break;
     case fPEEK:
         str = a1->pointer;
         value = peek (str);
@@ -935,7 +981,7 @@ static void
 store_buff (char *buff, int len)	/* store system-input buffer */
 {
     *buffcurr = my_malloc (sizeof (struct buff_chain));
-    memcpy ((*buffcurr)->buff, buff, SYSBUFFLEN + 1);
+    memcpy ((*buffcurr)->buff, buff, PIPEBUFFLEN + 1);
     (*buffcurr)->len = len;
     (*buffcurr)->next = NULL;
     buffcurr = &((*buffcurr)->next);
@@ -950,11 +996,11 @@ recall_buff ()			/* recall store buffer */
     char *result;
     int done, len;
 
-    result = (char *) my_malloc (buffcount * (SYSBUFFLEN + 1));
+    result = (char *) my_malloc (buffcount * (PIPEBUFFLEN + 1));
     curr = buffroot;
     len = 0;
     for (done = 0; done < buffcount && curr; done++) {
-        memcpy (result + len, curr->buff, SYSBUFFLEN);
+        memcpy (result + len, curr->buff, PIPEBUFFLEN);
         len += curr->len;
         old = curr;
         curr = curr->next;
@@ -967,7 +1013,7 @@ recall_buff ()			/* recall store buffer */
 static char *
 do_system2 (char *cmd)		/* hand over execution of command to system, return output as string */
 {
-    static char buff[SYSBUFFLEN + 1];	/* buffer to store command */
+    static char buff[PIPEBUFFLEN + 1];	/* buffer to store command */
     int len;			/* number of bytes read */
 #ifdef UNIX
     FILE *p;			/* points to pipe */
@@ -993,7 +1039,7 @@ do_system2 (char *cmd)		/* hand over execution of command to system, return outp
     }
     do {
         len = 0;
-        while (len < SYSBUFFLEN) {
+        while (len < PIPEBUFFLEN) {
             c = fgetc (p);
             if (c == EOF) {
                 buff[len] = '\0';
@@ -1038,7 +1084,7 @@ do_system2 (char *cmd)		/* hand over execution of command to system, return outp
 
     do {
         /* wait for output to arrive */
-        if (!ReadFile (piperead, buff, SYSBUFFLEN, (LPDWORD) & len, NULL)) {
+        if (!ReadFile (piperead, buff, PIPEBUFFLEN, (LPDWORD) & len, NULL)) {
             ret = GetLastError ();
         } else {
             ret = 0;
@@ -1539,11 +1585,12 @@ peek (char *dest)		/* peek into internals */
         return errorcode;
     } else if (!strcmp (dest, "isbound")) {
         return is_bound;
+    } else if (!strcmp (dest, "last_foreign_function_call_okay") || !strcmp (dest, "last_frnfn_call_okay")) {
+        return (double) last_frnfn_call_okay;
     } else if (!strcmp (dest, "secondsrunning")) {
-	time(&now);
-	return now-compilation_start;
+	return (double)((current_millis () - compilation_start)/1000);
     } else if (!strcmp (dest, "millisrunning")) {
-	return current_millis() - millis_compilation_start;
+	return (double)(current_millis () - compilation_start);
     } else if (dest[0] == '#') {
         error (sERROR, "don't use quotes when peeking into a file");
         return 0;
@@ -1596,7 +1643,7 @@ peek2 (char *dest, struct command *curr)	/* peek into internals */
     } else if (!strcmp (dest, "windoworigin")) {
         return my_strdup (winorigin);
     } else if (!strcmp (dest, "error")) {
-        return my_strdup (errorstring);
+        return my_strdup (estring);
     } else if (!strcmp (dest, "program_file_name")) {
         return my_strdup(main_file_name);
     } else if (!strcmp (dest, "program_name")) {
@@ -1615,6 +1662,8 @@ peek2 (char *dest, struct command *curr)	/* peek into internals */
 #endif
     } else if (!strcmp (dest, "font")) {
         return my_strdup (fontname);
+    } else if (!strcmp (dest, "last_foreign_function_call_error_text") || !strcmp (dest, "last_frnfn_call_error_text")) {
+	return my_strdup (last_frnfn_call_error_text);
     } else if (!strcmp (dest, "argument") || !strcmp (dest, "arguments")) {
         if (yabargc > 0) {
             s = yabargv[0];
