@@ -28,7 +28,6 @@ extern int switch_id;
 /* ------------- local functions ---------------- */
 
 static struct symbol *create_symbol (int, char *);	/* create a new symbol */
-static void stackdesc (int, char *);	/* give back string describing stackentry */
 static void freesym (struct symbol *);	/* free contents of symbol */
 static int ind_to_off (int *, int *);	/* convert array of indices to single offset */
 static void off_to_ind (int, int *, int *);	/* convert a single offset to an array of indices */
@@ -340,13 +339,12 @@ swap ()				/* swap topmost elements on stack */
 
 struct stackentry *
 push ()
-/* push element on stack and enlarge stack it */
+/* push element on stack and enlarge it */
 {
     struct stackentry *new;
 
     if (!stackhead->next) {
-        /* no next element */
-        /* create new element */
+        /* no next element, so create new element */
         new = (struct stackentry *) my_malloc (sizeof (struct stackentry));
         /* and initialize it */
         new->next = NULL;
@@ -395,7 +393,7 @@ pop (int etype)
     /* expected and found don't match */
     stackdesc (etype, expected);
     stackdesc (ftype, found);
-    sprintf (string, "expected '%s' but found '%s'", expected, found);
+    sprintf (string, "pop from stack: expected '%s' but found '%s'", expected, found);
     if (etype == stNUMBER || etype == stSTRING
             || etype == stSTRING_OR_NUMBER) {
         s = push ();
@@ -415,7 +413,7 @@ pop (int etype)
 }
 
 
-static void
+void
 stackdesc (int type, char *desc)	/* give back string describing stackentry */
 {
     switch (type) {
@@ -1044,9 +1042,18 @@ dim (struct command *cmd)	/* get room for array */
     ntotal = 1;
     for (i = 0; i < nar->dimension; i++) {
         (nar->bounds)[i] = nbounds[i];
+	if (ntotal > INT_MAX/nbounds[i]) {
+	    sprintf (string, "array index %d would cause the total size to overflow",i);
+            error (sERROR, string);
+            return;
+	}
         ntotal *= nbounds[i];
     }
     esize = (nar->type == 's') ? sizeof (char *) : sizeof (double);	/* size of one array element */
+    if (ntotal > INT_MAX/esize) {
+	error (sERROR, "array too large");
+	return;
+    }
     nar->pointer = my_malloc (ntotal * esize);
 
     if (oar) {
