@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2017
+    written by Marc Ihm 1995-2019
     more info at www.yabasic.de
 
     yabasic.h --- function prototypes and global variables
@@ -23,8 +23,7 @@
 */
 
 #if defined(UNIX) && defined(WINDOWS)
-UNIX and WINDOWS are defined at once;
-check your compiler settings
+#error UNIX and WINDOWS are defined together; check your compiler settings
 #endif
 /* ------------- includes ---------------- */
 #include <config.h>
@@ -64,10 +63,10 @@ check your compiler settings
 #include <X11/keysymdef.h>
 #include <unistd.h>
 #include <fcntl.h>
-#ifdef HAVE_NCURSES_HEADER
+#ifdef HAVE_NCURSES_H
 #include <ncurses.h>
 #else
-#ifdef HAVE_CURSES_HEADER
+#ifdef HAVE_CURSES_H
 #include <curses.h>
 #endif
 #endif
@@ -89,7 +88,9 @@ check your compiler settings
 #define OPEN_HAS_MODE 2
 #define OPEN_PRINTER 8
 #define STDIO_STREAM 1234
+
 /* -------- variables needed in all files and defined in ... -------- */
+
 /* main.c */
 extern struct command *current;	/* currently executed command */
 extern struct command *cmdroot;	/* first command */
@@ -99,13 +100,13 @@ extern int severity_threshold;  /* minimum severity the user wants to see */
 extern int severity_so_far;     /* maximum severity that has been printed until now */
 extern int interactive;		/* true, if commands come from stdin */
 extern char *progname;		/* name of yabasic-program */
-extern char *explanation[];	/* explanations of commands */
+extern char *cexplanation[];	/* explanations of commands */
+extern char *fexplanation[];	/* explanations of functions */
 extern char **yabargv;		/* arguments for yabasic */
 extern int yabargc;		/* number of arguments in yabargv */
-extern time_t compilation_start, compilation_end, execution_end;
-extern long long int millis_compilation_start;
+extern long long int compilation_start, compilation_end, execution_end;
 extern char *string;		/* for trash-strings */
-extern char *errorstring;	/* for error-strings */
+extern char *estring;	/* for error-strings */
 extern int errorcode;		/* error-codes */
 extern char library_path[];	/* full path to search libraries */
 extern int program_state;	/* state of program */
@@ -139,6 +140,10 @@ extern int winpid;		/* pid of process waiting for window keys */
 extern int termpid;		/* pid of process waiting for terminal keys */
 #endif
 
+/* foreign.c */
+extern char last_frnfn_call_error_text[]; /* last error message produced by foreign call */
+extern int last_frnfn_call_okay;         /* true, if last foreign call has been okay */
+
 /* graphic.c */
 /* printing and plotting */
 extern int print_to_file;	/* print to file ? */
@@ -188,7 +193,6 @@ extern BOOL Commandline;	/* true if launched from command line */
 extern int backpid;		/* pid of process waiting for redraw events */
 #endif
 
-
 /* function.c */
 extern struct command *datapointer;	/* current location for read-command */
 extern char *last_inkey; /* last result of inkey */
@@ -206,7 +210,7 @@ extern int labelcount;		/* count self-generated labels */
 extern struct library *library_stack[];	/* stack for library file names */
 extern struct library *currlib;	/* current libfile as relevant to bison */
 extern int inlib;		/* true, while in library */
-extern int fi_pending;		/* true, if within a short if */
+extern int in_short_if;		/* true, if within a short if */
 extern int library_chain_length;	/* length of library_chain */
 extern struct library *library_chain[];	/* list of all library file names */
 extern int include_depth; /* current position in libfile_stack */
@@ -216,7 +220,9 @@ extern char *current_function;	/* name of currently parsed function */
 extern int yydebug;
 extern int missing_endif;
 extern int missing_endif_line;
-void report_if_missing(int,char *,int);
+void report_if_missing(char *,int);
+void report_conflicting_close(char *,int);
+void collect_missing_clauses(char *, char);
     
 /*-------------------------- defs and undefs ------------------------*/
 
@@ -242,8 +248,9 @@ void report_if_missing(int,char *,int);
 #endif
 
 /* length of buffers for system() and input */
-#define SYSBUFFLEN 100
-#define INBUFFLEN 10000
+#define PIPEBUFFLEN 1000   /* writing into pipes, e.g. for system$ */
+#define INBUFFLEN 10000 /* user input */
+#define NAMEBUFFLEN 200 /* names chosen by user */
 
 
 /* ---------------------- enum types ------------------------------- */
@@ -265,7 +272,10 @@ enum stream_modes {
 
 enum functions {
     /* functions in yabasic (sorted by number of arguments) */
+    fFIRST_FUNCTION,		/* no function, just marks start of list */
     fRAN2, fDATE, fTIME,
+    fFRNFN_CALL, fFRNFN_CALL2, fFRNBF_ALLOC, fFRNBF_DUMP, fFRNBF_DUMP2,
+    fFRNBF_GET_NUMBER, fFRNBF_GET_STRING, fFRNBF_GET_BUFFER, fFRNFN_SIZE, fFRNBF_SIZE,
     fZEROARGS,
     fINKEY, fMOUSEX, fMOUSEY, fMOUSEB, fMOUSEMOD,
     fSIN, fASIN, fCOS, fACOS, fTAN,
@@ -273,13 +283,15 @@ enum functions {
     fSTR,
     fSQRT, fSQR, fFRAC, fABS, fSIG, fRAN, fINT, fCEIL, fFLOOR, fVAL, fASC, fHEX, fBIN, fDEC,
     fUPPER, fLOWER, fCHOMP, 
-    fLTRIM, fRTRIM, fTRIM, fCHR, fONEARGS, fDEC2, fATAN2, fLEFT, fAND, fOR,
+    fLTRIM, fRTRIM, fTRIM, fCHR,
+    fONEARGS, fDEC2, fATAN2, fLEFT, fAND, fOR,
     fEOR, fLOG2,
     fRIGHT, fINSTR, fRINSTR, fSTR2, fMOD, fMIN, fMAX, fPEEK3, fMID2,
     fTWOARGS,
     fMID, fINSTR2, fRINSTR2, fSTR3,
     fTHREEARGS,
-    fGETBIT, fGETCHAR
+    fGETBIT, fGETCHAR,
+    fLAST_FUNCTION,		/* no function, just marks end of list */
 };
 
 enum arraymode {
@@ -337,6 +349,9 @@ enum cmd_type {
     cOPENPRN, cCLOSEPRN, cMOVEORIGIN, cRECT, cGCOLOUR, cGCOLOUR2,
     cGBACKCOLOUR, cGBACKCOLOUR2, cPUTBIT, cPUTCHAR,
 
+    cFRNBF_SET_NUMBER, cFRNBF_SET_STRING, cFRNBF_FREE,    /* foreign libraries */
+    cFRNBF_SET_BUFFER,
+    
     cLAST_COMMAND			/* no command, just marks end of list */
 };
 
@@ -365,7 +380,7 @@ enum addmodes {
 
 enum states {
     /* current state of program */
-    HATCHED, INITIALIZED, COMPILING, RUNNING, FINISHED
+    spHATCHED, spINITIALIZED, spCOMPILING, spRUNNING, spFINISHED
 };
 
 enum yabkeys {
@@ -457,7 +472,7 @@ struct array {
 
 struct buff_chain {
     /* buffer chain for system-input */
-    char buff[SYSBUFFLEN + 1];	/* content of buffer */
+    char buff[PIPEBUFFLEN + 1];	/* content of buffer */
     int len;			/* used length of buff */
     struct buff_chain *next;	/* next buffer in chain */
 };
@@ -495,7 +510,6 @@ char *my_strndup (char *, int);	/*  own version of strndup */
 struct library *new_library (char *, char *);	/* create a new structure for library */
 char *dotify (char *, int);	/* add library name, if not already present */
 char *strip (char *);		/* strip off to minimal name */
-void do_error (struct command *);	/* issue user defined error */
 void create_docu (char *);	/* create command 'docu' */
 extern void add_variables (char *);	/* add pi and e to symbol table */
 void compile (void);		/* create a subroutine at runtime */
@@ -560,6 +574,20 @@ LRESULT CALLBACK mywindowproc (HWND, unsigned, UINT, DWORD);
 void calc_psscale (void);	/* calculate scale-factor for postscript */
 #endif
 
+/* foreign.c */
+void frnfn_call (int,double *,char **);
+double frnfn_size (void);
+char *frnbf_alloc (void);
+void frnbf_free (void);
+char *frnbf_dump (int);
+int frnbf_size ();
+void frnbf_set (void);
+void frnbf_set2 (void);
+double frnbf_get (void);
+char *frnbf_get2 (void);
+char *frnbf_get_buffer (void);
+void frnbf_set_buffer (void);
+
 /* function.c */
 void create_exception (int);	/* create command 'exception' */
 void exception (struct command *);	/* change handling of exceptions */
@@ -637,6 +665,7 @@ void dump_sym ();		/* dump the stack of lists of symbols */
 void dump_sub (int);		/* dump the stack of subroutine calls */
 void function_or_array (struct command *);	/* decide whether to do perform function or array */
 int count_args (int);	/* count number of arguments on stack */
+void stackdesc (int, char *);	/* give back string describing stackentry */
 
 
 /* flow.c */
